@@ -63,6 +63,7 @@ typedef enum {
    TEST_UPS,            /* TEST UPS Driver      */
    PCNET_UPS,           /* PCNET UPS Driver     */
    SNMPLITE_UPS,        /* SNMP Lite UPS Driver */
+   MODBUS_UPS,          /* MODBUS UPS Driver    */
 } UpsMode;
 
 typedef enum {
@@ -136,6 +137,7 @@ typedef struct internalgeninfo {
    int type;
 } INTERNALGENINFO;                 /* for assigning into upsinfo */
 
+class UpsDriver;
 
 class UPSINFO {
  public:
@@ -144,7 +146,6 @@ class UPSINFO {
    void clear_boost() { Status &= ~UPS_boost; };
    void clear_calibration() { Status &= ~UPS_calibration; };
    void clear_commlost() { Status &= ~UPS_commlost; };
-   void clear_dev_setup() { Status &= ~UPS_dev_setup; };
    void clear_fastpoll() { Status &= ~UPS_fastpoll; };
    void clear_onbatt_msg() { Status &= ~UPS_onbatt_msg; };
    void clear_onbatt() { Status &= ~UPS_onbatt; };
@@ -168,8 +169,8 @@ class UPSINFO {
    void set_boost() { Status |= UPS_boost; };
    void set_boost(int val) { if (val) set_boost(); else clear_boost(); };
    void set_calibration() { Status |= UPS_calibration; };
+   void set_calibration(int val) { if (val) set_calibration(); else clear_calibration(); };
    void set_commlost() { Status |= UPS_commlost; };
-   void set_dev_setup() { Status |= UPS_dev_setup; };
    void set_fastpoll() { Status |= UPS_fastpoll; };
    void set_onbatt_msg() { Status |= UPS_onbatt_msg; };
    void set_onbatt() { Status |= UPS_onbatt; };
@@ -202,7 +203,6 @@ class UPSINFO {
    bool is_boost() const { return (Status & UPS_boost) == UPS_boost; };
    bool is_calibration() const { return (Status & UPS_calibration) == UPS_calibration; };
    bool is_commlost() const { return (Status & UPS_commlost) == UPS_commlost; };
-   bool is_dev_setup() const { return (Status & UPS_dev_setup) == UPS_dev_setup; };
    bool is_fastpoll() const { return (Status & UPS_fastpoll) == UPS_fastpoll; };
    bool is_onbatt() const { return (Status & UPS_onbatt) == UPS_onbatt; };
    bool is_onbatt_msg() const { return (Status & UPS_onbatt_msg) == UPS_onbatt_msg; };
@@ -354,7 +354,7 @@ class UPSINFO {
    pthread_mutex_t mutex;
    int refcnt;                     /* thread attach count */
 
-   const struct upsdriver *driver; /* UPS driver for this UPSINFO */
+   UpsDriver *driver;              /* UPS driver for this UPSINFO */
    void *driver_internal_data;     /* Driver private data */
 };
 
