@@ -41,15 +41,13 @@ SnmpEngine::SnmpEngine() :
 
 SnmpEngine::~SnmpEngine()
 {
-   close(_socket);
-   close(_trapsock);
+   Close();
 }
 
-bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm, bool trap)
+bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm)
 {
    // In case we are already open
-   close(_socket);
-   close(_trapsock);
+   Close();
 
    // Remember new community name
    _community = comm;
@@ -100,29 +98,35 @@ bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm, b
    if (rc == -1)
    {
       perror("bind");
+      close(_socket);
+      _socket = -1;
       return false;
    }
 
-   // Open socket for receiving traps, if clients wants one
-   if (trap)
-   {
-      _trapsock = socket(PF_INET, SOCK_DGRAM, 0);
-      if (_trapsock == -1)
-      {
-         perror("socket");
-         return false;
-      }
+   return true;
+}
 
-      memset(&addr, 0, sizeof(addr));
-      addr.sin_family = AF_INET;
-      addr.sin_port = htons(SNMP_TRAP_PORT);
-      addr.sin_addr.s_addr = INADDR_ANY;
-      rc = bind(_trapsock, (struct sockaddr*)&addr, sizeof(addr));
-      if (rc == -1)
-      {
-         perror("bind");
-         return false;
-      }
+bool SnmpEngine::EnableTraps()
+{
+   _trapsock = socket(PF_INET, SOCK_DGRAM, 0);
+   if (_trapsock == -1)
+   {
+      perror("socket");
+      return false;
+   }
+
+   struct sockaddr_in addr;
+   memset(&addr, 0, sizeof(addr));
+   addr.sin_family = AF_INET;
+   addr.sin_port = htons(SNMP_TRAP_PORT);
+   addr.sin_addr.s_addr = INADDR_ANY;
+   int rc = bind(_trapsock, (struct sockaddr*)&addr, sizeof(addr));
+   if (rc == -1)
+   {
+      perror("bind");
+      close(_trapsock);
+      _trapsock = -1;
+      return false;
    }
 
    return true;
@@ -130,10 +134,17 @@ bool SnmpEngine::Open(const char *host, unsigned short port, const char *comm, b
 
 void SnmpEngine::Close()
 {
-   close(_socket);
-   _socket = -1;
-   close(_trapsock);
-   _trapsock = -1;
+   if (_socket != -1)
+   {
+      close(_socket);
+      _socket = -1;
+   }
+
+   if (_trapsock != -1)
+   {
+      close(_trapsock);
+      _trapsock = -1;
+   }
 }
 
 bool SnmpEngine::Set(const int oid[], Variable *data)
@@ -188,9 +199,15 @@ bool SnmpEngine::Get(alist<OidVar> &oids)
    // Append one varbind for each oidvar from the caller
    alist<OidVar>::iterator iter;
    for (iter = oids.begin(); iter != oids.end(); ++iter)
+   {
       if (iter->data.type != Asn::SEQUENCE)
          getreq.Append(iter->oid);
 
+      // Also use this loop as an opportunity to initialize all variables
+      // to invalid. They will be set to valid below as we fill in results.
+      iter->data.valid = false;
+   }
+
    // Perform request if we put at least one OID in it
    if (getreq.Size() > 0)
    {
@@ -265,6 +282,7 @@ bool SnmpEngine::Get(alist<OidVar> &oids)
             Variable tmp;
             result.Extract(&tmp);
             iter->data.seq.append(tmp);
+            iter->data.valid = true;
 
             // Save returned OID for next iteration
             nextoid = result.Oid();
@@ -483,10 +501,12 @@ bool VarBind::Extract(Variable *out)
    {
       out->i32 = _data->AsInteger()->IntValue();
       out->u32 = _data->AsInteger()->UintValue();
+      out->valid = true;
    }
    else if (_data->IsOctetString())
    {
       out->str = *_data->AsOctetString();
+      out->valid = true;
    }
    else
    {
