@@ -188,7 +188,7 @@ int usb_find_devices(void)
       // Populate device structure
       struct usb_device *dev = devices+ndevs;
       memset(dev, 0, sizeof(*dev));
-      astrncpy(dev->filename, detailData->DevicePath, sizeof(dev->filename));
+      strlcpy(dev->filename, detailData->DevicePath, sizeof(dev->filename));
       dev->bus = &bus;
       if (ndevs)
       {
@@ -259,6 +259,11 @@ int usb_claim_interface(usb_dev_handle *dev, int iface)
    return 0;
 }
 
+int usb_release_interface(usb_dev_handle *dev, int iface)
+{
+   return 0;
+}
+
 int usb_control_msg(usb_dev_handle *dev, int requesttype, 
                     int request, int value, int index, 
                     char *bytes, int size, int timeout)
