@@ -27,13 +27,13 @@ extern int ApcupsdMain(int argc, char **argv);
 #include "compat.h"
 
 // Standard command-line flag definitions
-#define ApcupsdRunService        "/service"
-#define ApcupsdRunAsUserApp      "/run"
-#define ApcupsdInstallService    "/install"
-#define ApcupsdRemoveService     "/remove"
-#define ApcupsdKillRunningCopy   "/kill"
-#define ApcupsdShowHelp          "/help"
-#define ApcupsdQuiet             "/quiet"
+char ApcupsdRunService[] =        "/service";
+char ApcupsdRunAsUserApp[] =      "/run";
+char ApcupsdInstallService[] =    "/install";
+char ApcupsdRemoveService[] =     "/remove";
+char ApcupsdKillRunningCopy[] =   "/kill";
+char ApcupsdShowHelp[] =          "/help";
+char ApcupsdQuiet[] =             "/quiet";
 
 // Usage string
 static const char *ApcupsdUsageText =
@@ -44,7 +44,8 @@ static HINSTANCE hAppInstance;
 
 // Command line argument storage
 #define MAX_COMMAND_ARGS 100
-static char *command_args[MAX_COMMAND_ARGS] = { "apcupsd", NULL };
+static char apcupsd_name[] = "apcupsd";
+static char *command_args[MAX_COMMAND_ARGS] = { apcupsd_name, NULL };
 static int num_command_args = 1;
 static char *winargs[MAX_COMMAND_ARGS];
 static int num_winargs = 0;
@@ -200,7 +201,8 @@ DWORD WINAPI EventThread(LPVOID param)
    // Create global exit event and allow Adminstrator access to it so any
    // member of the Administrators group can signal it.
    exitevt = CreateEvent(NULL, TRUE, FALSE, APCUPSD_STOP_EVENT_NAME);
-   GrantAccess(exitevt, EVENT_MODIFY_STATE, TRUSTEE_IS_GROUP, "Administrators");
+   TCHAR name[] = "Administrators";
+   GrantAccess(exitevt, EVENT_MODIFY_STATE, TRUSTEE_IS_GROUP, name);
 
    // Wait for event to be signaled
    while (runthread)
